@@ -185,6 +185,10 @@ struct msgb *ranap_new_msg_dt(uint8_t sapi, const uint8_t *nas, unsigned int nas
 
 	/* ies -> dt */
 	rc = ranap_encode_directtransferies(&dt, &ies);
+	if (rc < 0) {
+		LOGP(DRANAP, LOGL_ERROR, "error encoding direct transfer IEs: %d\n", rc);
+		return NULL;
+	}
 
 	/* dt -> msg */
 	msg = ranap_generate_initiating_message(RANAP_ProcedureCode_id_DirectTransfer,
@@ -251,6 +255,11 @@ struct msgb *ranap_new_msg_sec_mod_cmd(const uint8_t *ik, const uint8_t *ck, enu
 	if (ck)
 		ASN_STRUCT_FREE_CONTENTS_ONLY(asn_DEF_RANAP_EncryptionInformation, &ies.encryptionInformation);
 
+	if (rc < 0) {
+		LOGP(DRANAP, LOGL_ERROR, "error encoding security mode command IEs: %d\n", rc);
+		return NULL;
+	}
+
 	/* out -> msg */
 	msg = ranap_generate_initiating_message(RANAP_ProcedureCode_id_SecurityModeControl,
 						RANAP_Criticality_reject,
@@ -271,7 +280,7 @@ struct msgb *ranap_new_msg_sec_mod_compl(
 	RANAP_SecurityModeCompleteIEs_t ies;
 	RANAP_SecurityModeComplete_t out;
 	struct msgb *msg;
-	int i, rc;
+	int rc;
 
 	memset(&ies, 0, sizeof(ies));
 	memset(&out, 0, sizeof(out));
@@ -282,6 +291,10 @@ struct msgb *ranap_new_msg_sec_mod_compl(
 
 	/* ies -> out */
 	rc = ranap_encode_securitymodecompleteies(&out, &ies);
+	if (rc < 0) {
+		LOGP(DRANAP, LOGL_ERROR, "error encoding security mode complete IEs: %d\n", rc);
+		return NULL;
+	}
 
 	/* out -> msg */
 	msg = ranap_generate_successful_outcome(RANAP_ProcedureCode_id_SecurityModeControl,
@@ -317,10 +330,14 @@ struct msgb *ranap_new_msg_common_id(const char *imsi)
 
 	/* ies -> out */
 	rc = ranap_encode_commonid_ies(&out, &ies);
+
 	/* release dynamic allocations attached to ies */
 	ASN_STRUCT_FREE_CONTENTS_ONLY(asn_DEF_RANAP_PermanentNAS_UE_ID, &ies.permanentNAS_UE_ID);
-	if (rc < 0)
+
+	if (rc < 0) {
+		LOGP(DRANAP, LOGL_ERROR, "error encoding common id IEs: %d\n", rc);
 		return NULL;
+	}
 
 	/* out -> msg */
 	msg = ranap_generate_initiating_message(RANAP_ProcedureCode_id_CommonID,
@@ -349,8 +366,10 @@ struct msgb *ranap_new_msg_iu_rel_cmd(const RANAP_Cause_t *cause_in)
 
 	/* ies -> out */
 	rc = ranap_encode_iu_releasecommandies(&out, &ies);
-	if (rc < 0)
+	if (rc < 0) {
+		LOGP(DRANAP, LOGL_ERROR, "error encoding release command IEs: %d\n", rc);
 		return NULL;
+	}
 
 	/* out -> msg */
 	msg = ranap_generate_initiating_message(RANAP_ProcedureCode_id_Iu_Release,
@@ -377,8 +396,10 @@ struct msgb *ranap_new_msg_iu_rel_compl(void)
 
 	/* ies -> out */
 	rc = ranap_encode_iu_releasecompleteies(&out, &ies);
-	if (rc < 0)
+	if (rc < 0) {
+		LOGP(DRANAP, LOGL_ERROR, "error encoding release complete IEs: %d\n", rc);
 		return NULL;
+	}
 
 	/* out -> msg */
 	msg = ranap_generate_successful_outcome(RANAP_ProcedureCode_id_Iu_Release,
@@ -434,11 +455,15 @@ struct msgb *ranap_new_msg_paging_cmd(const char *imsi, const uint32_t *tmsi, in
 
 	/* ies -> out */
 	rc = ranap_encode_pagingies(&out, &ies);
+
 	/* release dynamic allocation attached to ies */
 	ASN_STRUCT_FREE_CONTENTS_ONLY(asn_DEF_RANAP_PermanentNAS_UE_ID, &ies.permanentNAS_UE_ID);
 	ASN_STRUCT_FREE_CONTENTS_ONLY(asn_DEF_RANAP_TemporaryUE_ID, &ies.temporaryUE_ID);
-	if (rc < 0)
+
+	if (rc < 0) {
+		LOGP(DRANAP, LOGL_ERROR, "error encoding paging IEs: %d\n", rc);
 		return NULL;
+	}
 
 	/* out -> msg */
 	msg = ranap_generate_initiating_message(RANAP_ProcedureCode_id_Paging,
@@ -638,6 +663,7 @@ static void new_transp_layer_addr(BIT_STRING_t *out, uint32_t ip, bool use_x213_
 {
 	uint8_t *buf;
 	unsigned int len;
+	uint32_t ip_h = ntohl(ip);
 
 	if (use_x213_nsap) {
 		len = 160/8;
@@ -645,11 +671,11 @@ static void new_transp_layer_addr(BIT_STRING_t *out, uint32_t ip, bool use_x213_
 		buf[0] = 0x35;	/* AFI For IANA ICP */
 		buf[1] = 0x00;	/* See A.5.2.1.2.7 of X.213 */
 		buf[2] = 0x01;
-		*(uint32_t *)&buf[3] = ntohl(ip);
+		memcpy(&buf[3], &ip_h, sizeof(ip_h));
 	} else {
-		len = 4;
+		len = sizeof(ip_h);
 		buf = CALLOC(len, sizeof(uint8_t));
-		*(uint32_t *)buf = ntohl(ip);
+		memcpy(buf, &ip_h, sizeof(ip_h));
 	}
 	out->buf = buf;
 	out->size = len;
@@ -716,7 +742,8 @@ static void assign_new_ra_id(RANAP_RAB_ID_t *id, uint8_t rab_id)
 
 /*! \brief generate RANAP RAB ASSIGNMENT REQUEST message for CS (voice).
  * See 3GPP TS 25.413 8.2.
- * RAB ID: 3GPP TS 25.413 9.2.1.2
+ * RAB ID: 3GPP TS 25.413 9.2.1.2.
+ * \param rtp_ip  MGW's RTP IPv4 address in *network* byte order.
  */
 struct msgb *ranap_new_msg_rab_assign_voice(uint8_t rab_id, uint32_t rtp_ip,
 					    uint16_t rtp_port,
@@ -785,7 +812,8 @@ struct msgb *ranap_new_msg_rab_assign_voice(uint8_t rab_id, uint32_t rtp_ip,
 	return msg;
 }
 
-/*! \brief generate RANAP RAB ASSIGNMENT REQUEST message for PS (data) */
+/*! \brief generate RANAP RAB ASSIGNMENT REQUEST message for PS (data)
+ * \param gtp_ip  SGSN's GTP IPv4 address in *network* byte order. */
 struct msgb *ranap_new_msg_rab_assign_data(uint8_t rab_id, uint32_t gtp_ip,
 					   uint32_t gtp_tei, bool use_x213_nsap)
 {
@@ -876,13 +904,17 @@ struct msgb *ranap_new_msg_iu_rel_req(const RANAP_Cause_t *cause)
 	memcpy(&ies.cause, cause, sizeof(ies.cause));
 
 	rc = ranap_encode_iu_releaserequesties(&out, &ies);
-	if (rc < 0)
+	if (rc < 0) {
+		LOGP(DRANAP, LOGL_ERROR, "error encoding release request IEs: %d\n", rc);
+		ASN_STRUCT_FREE_CONTENTS_ONLY(asn_DEF_RANAP_Iu_ReleaseRequest, &out);
 		return NULL;
+	}
 
 	/* encode the output into the msgb */
 	msg = ranap_generate_initiating_message(RANAP_ProcedureCode_id_Iu_ReleaseRequest,
 						RANAP_Criticality_reject,
 						&asn_DEF_RANAP_Iu_ReleaseRequest, &out);
+
 	ASN_STRUCT_FREE_CONTENTS_ONLY(asn_DEF_RANAP_Iu_ReleaseRequest, &out);
 
 	return msg;
@@ -912,11 +944,15 @@ struct msgb *ranap_new_msg_rab_rel_req(uint8_t rab_id, const RANAP_Cause_t *caus
 
 	/* encoe the list IEs into the output */
 	rc = ranap_encode_rab_releaserequesties(&out, &ies);
-	if (rc < 0)
-		return NULL;
+
 	/* 'out' has been generated, we can release the input */
 	ASN_STRUCT_FREE_CONTENTS_ONLY(asn_DEF_RANAP_RAB_ReleaseList, &ies.raB_ReleaseList);
 
+	if (rc < 0) {
+		LOGP(DRANAP, LOGL_ERROR, "error encoding release request IEs: %d\n", rc);
+		return NULL;
+	}
+
 	/* encode the output into the msgb */
 	msg = ranap_generate_initiating_message(RANAP_ProcedureCode_id_RAB_ReleaseRequest,
 						RANAP_Criticality_reject,
